﻿# -*- coding: UTF-8 -*-

# Save log: A global plugin for saving log files
# Version: 4.0
# It's possible to view the previous log
# Version: 3.0
# Now you can initialize the logging process
# Version: 2.3
# Now you can change menu position even when running NVDA from read only devices (wrong information: fixed in next version)
# Date: 12/09/2012

import addonHandler
import globalPluginHandler
import globalVars
import logHandler
import codecs
from logHandler import log
import languageHandler
import config
import os
import shutil
import wx
import gui
from gui.logViewer import LogViewer
import ui
from cStringIO import StringIO
from configobj import ConfigObj
from validate import Validator

addonHandler.initTranslation()

config.initConfigPath()
configPath = globalVars.appArgs.configPath
curLogFile = globalVars.appArgs.logFileName
prevLogFile = os.path.join(os.path.dirname(curLogFile), "nvda-old.log")
iniFileName = os.path.join(os.path.dirname(__file__), "saveLog.ini")

confspec = ConfigObj(StringIO("""#Configuration file

[menu]
	menuPosition = integer(default=0)
"""), encoding="UTF-8", list_values=False)
confspec.newlines = "\r\n"
conf = ConfigObj(iniFileName, configspec = confspec, indent_type = "\t", encoding="UTF-8")
val = Validator()
conf.validate(val)
pos = conf["menu"]["menuPosition"]

oldLogViewer = None

def activatePrevLogViewer():
	"""Activate the previous log viewer.
	If this log viewer has not already been created and opened, this will create and open it.
	Otherwise, it will be brought to the foreground if possible.
	"""
	global oldLogViewer
	if globalVars.appArgs.secure:
		# The log might expose sensitive information and the Save As dialog in the Log Viewer is a security risk.
		return
	if not oldLogViewer:
		oldLogViewer = PrevLogViewer(gui.mainFrame)
	oldLogViewer.Raise()
	# There is a MAXIMIZE style which can be used on the frame at construction, but it doesn't seem to work the first time it is shown,
	# probably because it was in the background.
	# Therefore, explicitly maximise it here.
	# This also ensures that it will be maximized whenever it is activated, even if the user restored/minimised it.
	oldLogViewer.Maximize()
	oldLogViewer.Show()

class PrevLogViewer(LogViewer):

	def __init__(self, parent):
		super(LogViewer, self).__init__(parent, wx.ID_ANY, _("NVDA Previous Log Viewer"))
		self.Bind(wx.EVT_ACTIVATE, self.onActivate)
		self.Bind(wx.EVT_CLOSE, self.onClose)
		mainSizer = wx.BoxSizer(wx.VERTICAL)
		self.outputCtrl = wx.TextCtrl(self, wx.ID_ANY, size=(500, 500), style=wx.TE_MULTILINE | wx.TE_READONLY|wx.TE_RICH)
		self.outputCtrl.Bind(wx.EVT_CHAR, self.onOutputChar)
		mainSizer.Add(self.outputCtrl, proportion=1, flag=wx.EXPAND)
		self.SetSizer(mainSizer)
		mainSizer.Fit(self)

		menuBar = wx.MenuBar()
		menu = wx.Menu()
		item = menu.Append(wx.ID_SAVEAS)
		self.Bind(wx.EVT_MENU, self.onSaveAsCommand, item)
		menu.AppendSeparator()
		item = menu.Append(wx.ID_EXIT, _("E&xit"))
		self.Bind(wx.EVT_MENU, self.onClose, item)
		menuBar.Append(menu, _("Previous log"))
		self.SetMenuBar(menuBar)
		self.refresh()
		self.outputCtrl.SetFocus()

	def refresh(self, evt=None):
		pos = self.outputCtrl.GetInsertionPoint()
		global prevLogFile
		# Append text to the output control which has been written to the old log file.
		try:
			f = codecs.open(prevLogFile, "r", encoding="UTF-8")
			self.outputCtrl.AppendText(f.read())
			self.outputCtrl.SetInsertionPoint(pos)
			f.close()
		except IOError:
			pass

	def onSaveAsCommand(self, evt):
		filename = wx.FileSelector(_("Save As"), default_filename="nvda-old.log", flags=wx.SAVE | wx.OVERWRITE_PROMPT, parent=self)
		if not filename:
			return
		try:
			# codecs.open() forces binary mode, which is bad under Windows because line endings won't be converted to crlf automatically.
			# Therefore, do the encoding manually.
			file(filename, "w").write(self.outputCtrl.GetValue().encode("UTF-8"))
		except (IOError, OSError), e:
			gui.messageBox(_("Error saving old log: %s") % e.strerror, _("Error"), style=wx.OK | wx.ICON_ERROR, parent=self)


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def __init__(self):
		super(globalPluginHandler.GlobalPlugin, self).__init__()
		if pos==1:
			self.menu = gui.mainFrame.sysTrayIcon.toolsMenu
		else:
			self.menu = gui.mainFrame.sysTrayIcon.menu
		self.saveLogMenu = wx.Menu()
		self.viewCurLogItem = self.saveLogMenu.Append(wx.ID_ANY, _("View &current log..."), _("View current log file"))
		MainFrame = gui.MainFrame()
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, MainFrame.onViewLogCommand, self.viewCurLogItem)
		self.mainItem = self.menu.AppendSubMenu(self.saveLogMenu, _("Save lo&g"), "Save log menu")
		self.viewOldLogItem = self.saveLogMenu.Append(wx.ID_ANY, _("&View previous log..."), _("View previous log file"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onViewOldLog, self.viewOldLogItem)
		self.initializeItem = self.saveLogMenu.Append(wx.ID_ANY, _("&Initialize logging..."), _("Delete log files"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onInitialize, self.initializeItem)
		self.settingsItem = self.saveLogMenu.Append(wx.ID_ANY, _("&Menu position..."), _("Choose menu position"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onSettings, self.settingsItem)
		self.aboutItem = self.saveLogMenu.Append(wx.ID_ANY, _("Open &documentation"), _("Open documentation for current language"))
		gui.mainFrame.sysTrayIcon.Bind(wx.EVT_MENU, self.onAbout, self.aboutItem)

	def terminate(self):
		try:
			self.menu.RemoveItem(self.mainItem)
		except wx.PyDeadObjectError:
			pass

	def onViewOldLog(self, evt):
		activatePrevLogViewer()

	def getDocFolder(self):
		langs = [languageHandler.getLanguage(), "en"]
		for lang in langs:
			docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", lang)
			if os.path.isdir(docFolder):
				return docFolder
			if "_" in lang:
				tryLang = lang.split("_")[0]
				docFolder = os.path.join(os.path.dirname(__file__), "..", "doc", tryLang)
				if os.path.isdir(docFolder):
					return docFolder
				if tryLang == "en":
					break
			if lang == "en":
				break
		return None

	def getDocPath(self, docFileName):
		docPath = self.getDocFolder()
		if docPath is not None:
			docFile = os.path.join(docPath, docFileName)
			if os.path.isfile(docFile):
				docPath = docFile
		return docPath

	def onAbout(self, evt):
		try:
			os.startfile(self.getDocPath("readme.html"))
		except WindowsError:
			pass

	def onInitialize(self, evt):
		if gui.messageBox(_("Are you sure you want to delete the content of NVDA's log files?"),
		_("Initialize logging process!"),
			wx.YES|wx.NO|wx.ICON_WARNING)==wx.YES:
			logHandler.initialize()

	def onSettings(self, evt):
		message = _("Choose the position for Save log menu")
		title = _("Save Log settings (2)")
		list = [_("Main menu - 1"), _("Tools menu - 2")]
		dlg = wx.SingleChoiceDialog(gui.mainFrame, message, title, choices=list)
		try:
			dlg.SetSelection(pos)
		except TypeError:
			dlg.SetSelection(0)
		gui.mainFrame.prePopup()
		try:
			result = dlg.ShowModal()
		except AttributeError:
			pass
		gui.mainFrame.postPopup()
		if result == wx.ID_OK:
			global pos
			oldPos = pos
			pos = dlg.GetSelection()
			if pos != oldPos:
				try:
					self.menu.RemoveItem(self.mainItem)
				except wx.PyDeadObjectError:
					pass
				self.__init__()
			conf["menu"]["menuPosition"] = pos
			try:
				conf.validate(val, copy=True)
				conf.write()
				log.info("SaveLog add-on configuration saved")
			except Exception, e:
				log.warning("Could not save SaveLog add-on configuration")
				log.debugWarning("", exc_info=True)
				raise e

	def script_saveLog(self, gesture):
		global curLogFile
		try:
			shutil.copy(curLogFile, configPath)
		except IOError:
			ui.message(_("Log file can not be copied"))
			return
		ui.message(_("Log file has been copied in NVDA's configuration folder"))
	script_saveLog.__doc__ = _("Copies log file to NVDA's configuration folder")

	__gestures = {"kb:control+shift+NVDA+l": "saveLog"}
